
ViewModelBase = class();

-- Initialize a new ViewModelBase instance
function ViewModelBase:Constructor()
end

-- Destroys all properties
function ViewModelBase:Destroy()
	self:RemoveCallbacks();

	for k, _ in pairs(self) do
		self[k] = nil;
	end
end

-- protected
-- Sets the property if different value and raises a property changed event
function ViewModelBase:SetProperty(propertyName, value)
	if self[propertyName] == value then
		return;
	end

	self[propertyName] = value;
	self:RaisePropertyChanged(propertyName);
end

-- protected
-- Raises a property changed event
function ViewModelBase:RaisePropertyChanged(propertyName)
	Events.FireEvent(self, self.PropertyChanged, propertyName);
end

function ViewModelBase:PropertyChanged(propertyName)
end

-- public
-- Adds a callback to the property changed event who is called when the property was changed
function ViewModelBase:OnPropertyChanged(propertyName, callback)
	self:AddCallback(self, "PropertyChanged", function(sender, name)
		if propertyName == name then
			callback(sender);
		end
	end);
end

-- protected
function ViewModelBase:AddCallback(object, event, callback)
	self._registrations = self._registrations or {};
	
	Events.AddCallback(object, event, callback);
	table.insert(self._registrations, {["object"] = object, ["event"] = event, ["callback"] = callback});
end

-- private
function ViewModelBase:RemoveCallbacks()
	if self._registrations then
		for _, reg in pairs(self._registrations) do
			Events.RemoveCallback(reg.object, reg.event, reg.callback);
		end

		self._registrations = nil;
	end
end
